// ignore_for_file: depend_on_referenced_packages

import 'dart:convert';
import 'dart:io';
import 'package:jobportal/data/company.dart';
import 'package:jobportal/data/jobseeker.dart';
import 'package:http/http.dart' as http;
import 'package:jobportal/logic/from_json.dart';
import 'package:jobportal/services/global_variables.dart';
import 'package:intl/intl.dart';
import '../data/company_job.dart';
import '../data/cv.dart';
import '../data/interview.dart';
import '../data/job.dart';
import '../data/job_application.dart';
import '../data/job_interview.dart';
import '../data/jobseeker_interview.dart';
import '../data/mynotification.dart';
import '../data/status.dart';

class Fetch {
  static Future<List<Jobseeker>> getAllApplicants(String id) async {
    final response = await http
        .get(Uri.parse('$myUrl/api/applicants/ApplicantsOfJob/$id'), headers: {
      HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
    });

    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<Jobseeker> jobseekers =
          jsonList.map((json) => FromJson.toJobseeker(json)).toList();
      return jobseekers;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<CV> getCvById(int id) async {
    print("5");
    final response = await http.get(Uri.parse('$myUrl/api/cvs/getById/$id'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });

    if (response.statusCode == 200) {
      print(response.body);
      final CV cv = FromJson.toCv(jsonDecode(response.body));

      return cv;
    } else {
      print(Exception().toString());
      throw Exception('Failed to load cv');
    }
  }

  static Future<Jobseeker> getJobseekerById(int id) async {
    print("5");
    final response = await http
        .get(Uri.parse('$myUrl/api/jobseekers/getById/$id'), headers: {
      HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
    });
    print(response.body.toString());
    if (response.statusCode == 200) {
      print(response.body);
      final Jobseeker jobseeker =
          FromJson.toJobseeker(jsonDecode(response.body));

      return jobseeker;
    } else {
      print(Exception().toString());
      throw Exception('Failed to load jobseeker');
    }
  }

  static Future<Company> getCompanyById(int id) async {
    final response = await http.get(Uri.parse('$myUrl/api/company/get/$id'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final Company company = FromJson.toCompany(jsonDecode(response.body));

      return company;
    } else {
      throw Exception('Failed to load company');
    }
  }

  static Future<int> getUnReadCompanyNotificationsNumber() async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/notifications/getUnReadNotificationNumForCompany/${identifier.id}'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final int number = json.decode(response.body);
      print("number: $number");
      return number;
    } else {
      print(response.body.toString());
      throw Exception('Failed to load number of notifications');
    }
  }

  static Future<int> getUnReadJobseekerNotificationsNumber() async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/notifications/getUnReadNotificationNumForJobseeker/${identifier.id}'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final int number = json.decode(response.body);
      print("number: $number");
      return number;
    } else {
      throw Exception('Failed to load number of notifications');
    }
  }

  static Future<Interview> checkIfBooked(String jobId, int jobseekerId) async {
    final response = await http.get(
        Uri.parse('$myUrl/api/interviews/checkIfBooked/$jobId/$jobseekerId'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    print(response.body);
    if (response.statusCode == 200) {
      return FromJson.toInterview(jsonDecode(response.body));
    } else {
      throw Exception();
    }
  }

  static Future<Status> checkIfHasCv(int jobseekerId) async {
    final response = await http.get(
        Uri.parse('http://192.168.96.89:8086/api/cvs/exist/$jobseekerId'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    return FromJson.toStatus(jsonDecode(response.body));
  }

  static Future<JobApplication?> checkIfApplied(
      String jobId, int jobseekerId) async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/applicants/GetByJobIdAndApplicantId/$jobId/$jobseekerId'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });

    if (response.statusCode == 200 || response.statusCode == 201) {
      if (jsonDecode(response.body) != null) {
        final jobApplication =
            FromJson.toJobApplication(jsonDecode(response.body));
        return jobApplication;
      } else {
        return null;
      }
    } else {
      throw Exception('Failed to check if applied: ${response.statusCode}');
    }
  }

  static Future<List<MyNotification>> getAllNotificationByCompanyId() async {
    print(identifier.id);
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/notifications/getAllByCompanyId/${identifier.id}'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });

    if (response.statusCode == 200) {
      print(response.body);
      final List<dynamic> jsonList = json.decode(response.body);
      final List<MyNotification> notificationDetails =
          jsonList.map((json) => FromJson.toMyNotification(json)).toList();
      print(notificationDetails);
      print(notificationDetails.length);

      return notificationDetails;
    } else {
      throw Exception('Failed to load notifications');
    }
  }

  static Future<List<MyNotification>> getAllNotificationByJobseekerId() async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/notifications/getAllByJobseekerId/${identifier.id}'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });

    if (response.statusCode == 200) {
      print(response.body);
      final List<dynamic> jsonList = json.decode(response.body);
      final List<MyNotification> notificationDetails =
          jsonList.map((json) => FromJson.toMyNotification(json)).toList();
      print(notificationDetails);
      print(notificationDetails.length);

      return notificationDetails;
    } else {
      throw Exception('Failed to load notifications');
    }
  }

  static Future<List<Job>> getAllJobs() async {
    final response = await http.get(Uri.parse('$myUrl/api/jobs/getAll'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      print(response.body.toString());
      final List<dynamic> jsonList = json.decode(response.body);
      final List<Job> jobDetails =
          jsonList.map((json) => FromJson.toJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<Job>> getAllJobsByPage(int pageNumber) async {
    final response = await http
        .get(Uri.parse('$myUrl/api/jobs/getAll/$pageNumber'), headers: {
      HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
    });
    if (response.statusCode == 200) {
      print(response.body.toString());
      final List<dynamic> jsonList = json.decode(response.body);
      final List<Job> jobDetails =
          jsonList.map((json) => FromJson.toJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<Job>> getAllJobsByJobseekerId(int id) async {
    final response = await http
        .get(Uri.parse('$myUrl/api/applicants/JobOfApplicants/$id'), headers: {
      HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
    });
    print(response.body.toString());
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<Job> jobDetails =
          jsonList.map((json) => FromJson.toJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<Job>> getAllJobsByCompanyName(String companyName) async {
    final response = await http.get(
        Uri.parse('$myUrl/api/jobs/getByCompanyName/$companyName'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<Job> jobDetails =
          jsonList.map((json) => FromJson.toJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<Job>> getAllJobsBySpecificationName(
      String specificationName) async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/jobs/getByJobSpecificationName/$specificationName/0'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<Job> jobDetails =
          jsonList.map((json) => FromJson.toJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<Job>> getAllJobsByTimeType(String timeType) async {
    final response = await http
        .get(Uri.parse('$myUrl/api/jobs/getByTimeType/$timeType'), headers: {
      HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
    });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<Job> jobDetails =
          jsonList.map((json) => FromJson.toJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<CompanyJob>> getJobsByCompanyId(int id) async {
    final response = await http.get(Uri.parse('$myUrl/api/jobs/company/${id}'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<CompanyJob> jobDetails =
          jsonList.map((json) => FromJson.toCompanyJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<CompanyJob>> getJobsByCompanyIdAndPage(
      int id, int paage) async {
    final response = await http
        .get(Uri.parse('$myUrl/api/jobs/company/${id}/${paage}'), headers: {
      HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
    });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<CompanyJob> jobDetails =
          jsonList.map((json) => FromJson.toCompanyJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<CompanyJob>> getJobsByCompanyIdAndJobName(
      int companyId, String jobName) async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/jobs/getJobsByCompanyIdAndJobName/$companyId/$jobName'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<CompanyJob> jobDetails =
          jsonList.map((json) => FromJson.toCompanyJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<CompanyJob>> getJobsByCompanyIdAndJobNameAndPageNumber(
      int companyId, String jobName, int pageNumber) async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/jobs/getJobsByCompanyIdAndJobName/$companyId/$jobName/$pageNumber'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<CompanyJob> jobDetails =
          jsonList.map((json) => FromJson.toCompanyJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<Job>> getJobsByJobseekerIdAndJobName(
      int jobseekerId, String jobName) async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/applicants/JobOfApplicantsByName/$jobseekerId/$jobName'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<Job> jobDetails =
          jsonList.map((json) => FromJson.toJob(json)).toList();
      return jobDetails;
    } else {
      throw Exception('Failed to load jobs');
    }
  }

  static Future<List<JobseekerInterview>> getAllInterviewsByJobseekerId(
      int id) async {
    final response = await http.get(
        Uri.parse('$myUrl/api/interviews/getAllByJobseekerId/$id'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<JobseekerInterview> interviews =
          jsonList.map((json) => FromJson.toJobseekerInterview(json)).toList();
      return interviews;
    } else {
      throw Exception('Failed to load jobseeker interviews');
    }
  }

  static Future<List<JobInterview>> getAllInterviewsByCompanyId(int id) async {
    final response = await http.get(
        Uri.parse('$myUrl/api/interviews/getAllByCompanyId/$id'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      final List<dynamic> jsonList = json.decode(response.body);
      final List<JobInterview> interviews =
          jsonList.map((json) => FromJson.toJobInterview(json)).toList();
      return interviews;
    } else {
      throw Exception('Failed to load jobseeker interviews');
    }
  }

  static Future<List<JobInterview>> getAllInterviewsByJobId(int id) async {
    final response = await http
        .get(Uri.parse('$myUrl/api/interviews/getAllByJobId/$id'), headers: {
      HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
    });
    if (response.statusCode == 200) {
      print(response.body.toString());
      final List<dynamic> jsonList = json.decode(response.body);
      final List<JobInterview> interviews =
          jsonList.map((json) => FromJson.toJobInterview(json)).toList();
      return interviews;
    } else {
      throw Exception('Failed to load job interviews');
    }
  }

  static Future<List<JobInterview>> getAllInterviewsByJobIdAndDate(
      int jobId, DateTime date) async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/interviews/getAllByJobIdAndDate/$jobId/${DateFormat('yyyy-MM-dd').format(date)}'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      print(response.body.toString());
      final List<dynamic> jsonList = json.decode(response.body);
      final List<JobInterview> interviews =
          jsonList.map((json) => FromJson.toJobInterview(json)).toList();
      return interviews;
    } else {
      throw Exception('Failed to load jobseeker interviews');
    }
  }

  static Future<List<JobInterview>> getAllInterviewsByCompanyIdAndDate(
      int companyId, DateTime date) async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/interviews/getAllByCompanyIdAndDate/$companyId/${DateFormat('yyyy-MM-dd').format(date)}'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      print(response.body.toString());
      final List<dynamic> jsonList = json.decode(response.body);
      final List<JobInterview> interviews =
          jsonList.map((json) => FromJson.toJobInterview(json)).toList();
      return interviews;
    } else {
      throw Exception('Failed to load jobseeker interviews');
    }
  }

  static Future<List<JobseekerInterview>> getAllInterviewsByJobseekerIdAndDate(
      int jobseekerId, DateTime date) async {
    final response = await http.get(
        Uri.parse(
            '$myUrl/api/interviews/getInterviewsByJobseekerIdAndDate/$jobseekerId/${DateFormat('yyyy-MM-dd').format(date)}'),
        headers: {
          HttpHeaders.authorizationHeader: "Bearer ${identifier.token}"
        });
    if (response.statusCode == 200) {
      print(response.body.toString());
      final List<dynamic> jsonList = json.decode(response.body);
      final List<JobseekerInterview> interviews =
          jsonList.map((json) => FromJson.toJobseekerInterview(json)).toList();
      return interviews;
    } else {
      throw Exception('Failed to load jobseeker interviews');
    }
  }
}
